@@ -20,18 +20,10 @@ import os
 import optparse
 from gettext import gettext as _
 
-# gtk
+import gio
 import gtk
-import gtk.glade
 import gobject
 
-# Drag'N'Drop support needs gnomevfs
-try:
-    import gnomevfs
-    gnomevfs_available = True
-except ImportError:
-    gnomevfs_available = False
-
 # project
 import paths
 from ui import gnomeglade
@@ -47,7 +39,7 @@ import vc
 
 from util.sourceviewer import srcviewer
 
-version = "1.3.3"
+version = "1.5.0"
 
 ################################################################################
 #
@@ -57,7 +49,7 @@ version = "1.3.3"
 
 class NewDocDialog(gnomeglade.Component):
     def __init__(self, parentapp):
-        gnomeglade.Component.__init__(self, paths.ui_dir("meldapp.glade"), "newdialog")
+        gnomeglade.Component.__init__(self, paths.ui_dir("meldapp.ui"), "newdialog")
         self.map_widgets_into_lists(["fileentry", "direntry", "vcentry", "three_way_compare"])
         self.entrylists = self.fileentry, self.direntry, self.vcentry
         self.widget.set_transient_for(parentapp.widget)
@@ -84,7 +76,7 @@ class NewDocDialog(gnomeglade.Component):
 
     def on_response(self, dialog, arg):
         if arg == gtk.RESPONSE_OK:
-            page = self.notebook.get_current_page()
+            page = self.type_notebook.get_current_page()
             paths = [e.get_full_path() or "" for e in self.entrylists[page]]
             if page < 2 and not self.three_way_compare[page].get_active():
                 paths.pop(0)
@@ -128,11 +120,12 @@ class MeldApp(gnomeglade.Component):
     # init
     #
     def __init__(self):
-        gladefile = paths.ui_dir("meldapp.glade")
+        gladefile = paths.ui_dir("meldapp.ui")
         gtk.window_set_default_icon_name("icon")
         if getattr(gobject, "pygobject_version", ()) >= (2, 16, 0):
             gobject.set_application_name("Meld")
         gnomeglade.Component.__init__(self, gladefile, "meldapp")
+        gtk.window_set_default_icon_name("meld")
         self.prefs = preferences.MeldPreferences()
 
         actions = (
@@ -155,8 +148,8 @@ class MeldApp(gnomeglade.Component):
             ("Preferences", gtk.STOCK_PREFERENCES, _("Prefere_nces"), None, _("Configure the application"), self.on_menu_preferences_activate),
 
             ("ChangesMenu", None, _("_Changes")),
-            ("NextChange", gtk.STOCK_GO_DOWN, _("Next change"), "<control>D", _("Go to the next change"), self.on_menu_edit_down_activate),
-            ("PrevChange", gtk.STOCK_GO_UP,   _("Previous change"), "<control>E", _("Go to the previous change"), self.on_menu_edit_up_activate),
+            ("NextChange", gtk.STOCK_GO_DOWN, _("Next change"), "<Alt>Down", _("Go to the next change"), self.on_menu_edit_down_activate),
+            ("PrevChange", gtk.STOCK_GO_UP,   _("Previous change"), "<Alt>Up", _("Go to the previous change"), self.on_menu_edit_up_activate),
 
             ("ViewMenu", None, _("_View")),
             ("FileStatus",  None, _("File status")),
@@ -192,6 +185,14 @@ class MeldApp(gnomeglade.Component):
         self.widget.add_accel_group(self.ui.get_accel_group())
         self.menubar = self.ui.get_widget('/Menubar')
         self.toolbar = self.ui.get_widget('/Toolbar')
+
+        # Add alternate keybindings for Prev/Next Change
+        accels = self.ui.get_accel_group()
+        (keyval, mask) = gtk.accelerator_parse("<Ctrl>D")
+        accels.connect_group(keyval, mask, 0, self.on_menu_edit_down_activate)
+        (keyval, mask) = gtk.accelerator_parse("<Ctrl>E")
+        accels.connect_group(keyval, mask, 0, self.on_menu_edit_up_activate)
+
         self.appvbox.pack_start(self.menubar, expand=False)
         self.appvbox.pack_start(self.toolbar, expand=False)
         # TODO: should possibly use something other than doc_status
@@ -201,8 +202,8 @@ class MeldApp(gnomeglade.Component):
             gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
             [ ('text/uri-list', 0, 0) ],
             gtk.gdk.ACTION_COPY)
-        if gnomevfs_available:
-            self.widget.connect('drag_data_received', self.on_widget_drag_data_received)
+        self.widget.connect("drag_data_received",
+                            self.on_widget_drag_data_received)
         self.toolbar.set_style( self.prefs.get_toolbar_style() )
         self.toolbar.props.visible = self.prefs.toolbar_visible
         self.status_box.props.visible = self.prefs.statusbar_visible
@@ -227,7 +228,9 @@ class MeldApp(gnomeglade.Component):
 
     def on_widget_drag_data_received(self, wid, context, x, y, selection_data, info, time):
         if len(selection_data.get_uris()) != 0:
-            paths = [gnomevfs.get_local_path_from_uri(u) for u in selection_data.get_uris()]
+            paths = []
+            for uri in selection_data.get_uris():
+                paths.append(gio.File(uri=uri).get_path())
             self.open_paths(paths)
             return True
 
@@ -316,9 +319,10 @@ class MeldApp(gnomeglade.Component):
                                            self.on_next_diff_changed)
         self.scheduler.add_task( newdoc.scheduler )
 
-    def on_notebook_label_changed(self, component, text):
+    def on_notebook_label_changed(self, component, text, tooltip):
         nbl = self.notebook.get_tab_label( component.widget )
         nbl.set_label_text(text)
+        nbl.set_tooltip_text(tooltip)
         self.widget.set_title(text + " - Meld")
         self.notebook.child_set_property(component.widget, "menu-label", text)
 
@@ -443,7 +447,11 @@ class MeldApp(gnomeglade.Component):
 
     def on_menu_about_activate(self, *extra):
         gtk.about_dialog_set_url_hook(lambda dialog, uri: misc.open_uri(uri))
-        about = gtk.glade.XML(paths.ui_dir("meldapp.glade"), "about").get_widget("about")
+        builder = gtk.Builder()
+        # FIXME: domain literal duplicated from bin/meld
+        builder.set_translation_domain("meld")
+        builder.add_objects_from_file(paths.ui_dir("meldapp.ui"), ["about"])
+        about = builder.get_object("about")
         about.props.version = version
         about.set_transient_for(self.widget)
         about.run()
@@ -492,10 +500,13 @@ class MeldApp(gnomeglade.Component):
         page.connect("create-diff", lambda obj,arg: self.append_diff(arg) )
         page.connect("status-changed", lambda junk,arg: self.statusbar.set_doc_status(arg) )
 
+        # Allow reordering of tabs
+        self.notebook.set_tab_reorderable(page.widget, True);
+
     def append_dirdiff(self, dirs, auto_compare=False):
         assert len(dirs) in (1,2,3)
         doc = dirdiff.DirDiff(self.prefs, len(dirs))
-        self._append_page(doc, "tree-folder-normal")
+        self._append_page(doc, "folder")
         doc.set_locations(dirs)
         # FIXME: This doesn't work, as dirdiff behaves differently to vcview
         if auto_compare:
@@ -512,7 +523,7 @@ class MeldApp(gnomeglade.Component):
         seq.clear()
         seq.connect("can-undo", self.on_can_undo)
         seq.connect("can-redo", self.on_can_redo)
-        self._append_page(doc, "tree-file-normal")
+        self._append_page(doc, "text-x-generic")
         doc.set_files(files)
         return doc
 
@@ -547,6 +558,7 @@ class MeldApp(gnomeglade.Component):
         assert len(locations) in (1,)
         location = locations[0]
         doc = vcview.VcView(self.prefs)
+        # FIXME: need a good themed VC icon
         self._append_page(doc, "vc-icon")
         doc.set_location(location)
         if auto_compare:
@@ -604,10 +616,9 @@ class MeldApp(gnomeglade.Component):
             help=_("Set label to use instead of file name"))
         parser.add_option("-a", "--auto-compare", action="store_true", default=False,
             help=_("Automatically compare all differing files on startup"))
-        parser.add_option("-u", "--unified", action="store_true", help=_("Ignored for compatibility"))
-        parser.add_option("-c", "--context", action="store_true", help=_("Ignored for compatibility"))
-        parser.add_option("-e", "--ed", action="store_true", help=_("Ignored for compatibility"))
-        parser.add_option("-r", "--recursive", action="store_true", help=_("Ignored for compatibility"))
+        parser.add_option("-o", "--output", action="store", type="string",
+            dest="outfile", default=None,
+            help=_("Set the target file for saving a merge result"))
         parser.add_option("", "--diff", action="callback", callback=self.diff_files_callback,
                           dest="diff", default=[],
                           help=_("Creates a diff tab for up to 3 supplied files or directories."))
@@ -616,12 +627,15 @@ class MeldApp(gnomeglade.Component):
             parser.error(_("too many arguments (wanted 0-4, got %d)") % len(args))
 
         for files in options.diff:
-            self.append_diff(files)
+            self.open_paths(files)
 
         tab = self.open_paths(args, options.auto_compare)
         if tab:
             tab.set_labels(options.label)
 
+        if options.outfile and tab and isinstance(tab, filediff.FileDiff):
+            tab.set_merge_output_file(options.outfile)
+
     def _single_file_open(self, path):
         doc = vcview.VcView(self.prefs)
         def cleanup():
