@@ -25,6 +25,7 @@ import misc
 from ui import gnomeglade
 import melddoc
 import paths
+import ui.emblemcellrenderer
 import vc
 
 ################################################################################
@@ -33,16 +34,6 @@ import vc
 #
 ################################################################################
 
-def _expand_to_root( treeview, path ):
-    """Expand rows from path up to root"""
-    start = path[:]
-    while len(start) and not treeview.row_expanded(start):
-        start = start[:-1]
-    level = len(start)
-    while level < len(path):
-        level += 1
-        treeview.expand_row( path[:level], 0)
-
 def _commonprefix(files):
     if len(files) != 1:
         workdir = misc.commonprefix(files)
@@ -57,7 +48,7 @@ def _commonprefix(files):
 ################################################################################
 class CommitDialog(gnomeglade.Component):
     def __init__(self, parent):
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.glade"), "commitdialog")
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"), "commitdialog")
         self.parent = parent
         self.widget.set_transient_for( parent.widget.get_toplevel() )
         selected = parent._get_selected_files()
@@ -88,7 +79,9 @@ COL_LOCATION, COL_STATUS, COL_REVISION, COL_TAG, COL_OPTIONS, COL_END = range(tr
 
 class VcTreeStore(tree.DiffTreeStore):
     def __init__(self):
-        tree.DiffTreeStore.__init__(self, 1, COL_END)
+        ntree = 1
+        types = [str] * COL_END * ntree
+        tree.DiffTreeStore.__init__(self, ntree, types)
         self.textstyle[tree.STATE_MISSING] = '<span foreground="#000088" strikethrough="true" weight="bold">%s</span>'
 
 ################################################################################
@@ -119,7 +112,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def __init__(self, prefs):
         melddoc.MeldDoc.__init__(self, prefs)
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.glade"), "vcview")
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"), "vcview")
 
         actions = (
             ("VcCompare",       gtk.STOCK_DIALOG_INFO,      _("_Compare"),      None, _("Compare selected"), self.on_button_diff_clicked),
@@ -161,12 +154,15 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         self.treeview.set_headers_visible(1)
         column = gtk.TreeViewColumn( _("Name") )
-        renpix = gtk.CellRendererPixbuf()
+        renicon = ui.emblemcellrenderer.EmblemCellRenderer()
         rentext = gtk.CellRendererText()
-        column.pack_start(renpix, expand=0)
+        column.pack_start(renicon, expand=0)
         column.pack_start(rentext, expand=1)
-        column.set_attributes(renpix, pixbuf=self.model.column_index(tree.COL_ICON, 0))
-        column.set_attributes(rentext, markup=self.model.column_index(tree.COL_TEXT, 0))
+        col_index = self.model.column_index
+        column.set_attributes(renicon,
+                              icon_name=col_index(tree.COL_ICON, 0),
+                              icon_tint=col_index(tree.COL_TINT, 0))
+        column.set_attributes(rentext, markup=col_index(tree.COL_TEXT, 0))
         self.treeview.append_column(column)
 
         def addCol(name, num):
@@ -203,10 +199,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         # VC ComboBox
         self.combobox_vcs = gtk.ComboBox()
         self.combobox_vcs.lock = True
-        self.combobox_vcs.set_model(gtk.ListStore(str, object))
+        self.combobox_vcs.set_model(gtk.ListStore(str, object, bool))
         cell = gtk.CellRendererText()
         self.combobox_vcs.pack_start(cell, False)
         self.combobox_vcs.add_attribute(cell, 'text', 0)
+        self.combobox_vcs.add_attribute(cell, 'sensitive', 2)
         self.combobox_vcs.lock = False
         self.hbox2.pack_end(self.combobox_vcs, expand=False)
         self.combobox_vcs.show()
@@ -229,15 +226,39 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.combobox_vcs.get_model().clear()
         tooltip_texts = [_("Choose one Version Control"),
                          _("Only one Version Control in this directory")]
-        default_active = 0
+        default_active = -1
+        valid_vcs = []
         # Try to keep the same VC plugin active on refresh()
         for idx, avc in enumerate(vcs):
-            if (self.vc is not None and
-                self.vc.__class__ == avc.__class__):
-                default_active = idx
-            self.combobox_vcs.get_model().append([avc.NAME, avc])
-        if gtk.pygtk_version >= (2, 12, 0):
-            self.combobox_vcs.set_tooltip_text(tooltip_texts[len(vcs) == 1])
+            # See if the necessary version control command exists.  If so,
+            # make sure what we're diffing is a valid respository.  If either
+            # check fails don't let the user select the that version control
+            # tool and display a basic error message in the drop-down menu.
+            err_str = ""
+            if vc._vc.call(["which", avc.CMD]):
+                # TRANSLATORS: this is an error message when a version control
+                # application isn't installed or can't be found
+                err_str = _("%s Not Installed" % avc.CMD)
+            elif not avc.valid_repo():
+                # TRANSLATORS: this is an error message when a version
+                # controlled repository is invalid or corrupted
+                err_str = _("Invalid Repository")
+            else:
+                valid_vcs.append(idx)
+                if (self.vc is not None and
+                     self.vc.__class__ == avc.__class__):
+                     default_active = idx
+
+            if err_str:
+                self.combobox_vcs.get_model().append( \
+                        [_("%s (%s)") % (avc.NAME, err_str), avc, False])
+            else:
+                self.combobox_vcs.get_model().append([avc.NAME, avc, True])
+
+        if valid_vcs and default_active == -1:
+            default_active = min(valid_vcs)
+
+        self.combobox_vcs.set_tooltip_text(tooltip_texts[len(vcs) == 1])
         self.combobox_vcs.set_sensitive(len(vcs) > 1)
         self.combobox_vcs.lock = False
         self.combobox_vcs.set_active(default_active)
@@ -270,6 +291,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def recompute_label(self):
         self.label_text = os.path.basename(self.location)
+        # TRANSLATORS: This is the location of the directory the user is diffing
+        self.tooltip_text = _("%s: %s") % (_("Location"), self.location)
         self.label_changed()
 
     def _search_recursively_iter(self, iterstart):
@@ -319,7 +342,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 if len(entries) == 0:
                     self.model.add_empty(it, _("(Empty)"))
                 if differences or len(path)==1:
-                    _expand_to_root( self.treeview, path )
+                    self.treeview.expand_to_path(path)
             else: # just the root
                 self.treeview.expand_row( (0,), 0)
         self.vc.uncache_inventory()
